App.StonehearthCitizensView = App.View.extend({
   templateName: 'citizens',
   uriProperty: 'model',
   closeOnEsc: true,
   components: {
      "citizens" : {
         "*": {
            "stonehearth:job": {
            },
            "unit_info": {},  
         }
      },
      "work_orders": {}
   },

   init: function() {
      var self = this;
      this._super();

      radiant.call('stonehearth:get_population')
         .done(function(response){
            self._populationUri = response.population;
            self.set('uri', self._populationUri);
         });
   },

   didInsertElement: function() {
      var self = this;  
      self._super();

      var stats = ["mind", "body", "spirit"];
      radiant.each(stats, function(i, stat) {
         var tooltipString = App.tooltipHelper.getTooltip(stat);
         self.$('#' + stat).tooltipster({content: $(tooltipString)});
      });
      self.$().on('click', ':checkbox', function() {
         var checked = $(this).is(':checked');
         var workOrder = $(this).attr('workOrder');
         var citizenId = $(this).attr('citizenId');
         radiant.call_obj(self._populationUri, 'change_work_order_command', workOrder, parseInt(citizenId), checked);
      });
   },

   citizensArray: function() {
      var self = this;
      var citizens = radiant.map_to_array(self.get('model.citizens'));
      citizens.sort(function(a, b){
         var aJobName = i18n.t(a['stonehearth:job'].curr_job_name);
         var bJobName = i18n.t(b['stonehearth:job'].curr_job_name);
         var n = aJobName.localeCompare(bJobName);
         if (n == 0) {
            var aName = i18n.t(a['unit_info'].display_name, {self: a});
            var bName = i18n.t(b['unit_info'].display_name, {self: b});
            n = aName.localeCompare(bName);
         }

         return n;
      });
      return citizens;
   }.property('model.citizens'),

   setSelectedCitizen: function(citizen) {
      var self = this;
      console.log(citizen["stonehearth:job"])
      var existingSelected = self.get('selected');
      if (citizen) {
         var uri = citizen.__self;
         if (!existingSelected || (uri != existingSelected.__self)) {
            var portrait_url = '/r/get_portrait/?type=headshot&animation=idle_breathe.json&entity=' + uri;
            self.$('#selectedPortrait').css('background-image', 'url(' + portrait_url + ')');
         }
      } else {
         self.$('#selectedPortrait').css('background-image', 'url()');
      }

      self.set('selected', citizen);
   },

   actions: {
      doCommand: function(command, citizen_id) {  
         App.stonehearthClient.doCommand(citizen_id, command);
      },
      showCharacterSheet: function(citizen_id) {
         App.stonehearthClient.showCharacterSheet(citizen_id);
         radiant.call('stonehearth:camera_look_at_entity', citizen_id);
         radiant.call('stonehearth:select_entity', citizen_id);
         radiant.call('radiant:play_sound', {'track' : 'stonehearth:sounds:ui:start_menu:focus' });
      },
      suspendToggle: function(workOrder) {
         var self = this;
         var work_orders = self.get('model.work_orders');
         if (!work_orders) {
            return;
         }
         var workOrderData = work_orders[workOrder];
         if (!workOrderData) {
            // why doesn't it exist?!
            return;
         }
         var isSuspended = !workOrderData.is_suspended;
         radiant.call_obj(self._populationUri, 'set_work_order_suspend_command', workOrder, isSuspended);
      }
   }
});

App.StonehearthCitizenTasksRowView = App.View.extend({
   tagName: 'tr',
   classNames: ['row'],
   templateName: 'citizenTasksRow',
   uriProperty: 'model',

   components: {
      "unit_info": {},           
      "stonehearth:commands": {
         commands: []
      },
      "stonehearth:ai": {
         "status_text_data" : {}
      },
      "stonehearth:job": {
      },
      "stonehearth:crafter": {
         "workshop": {}
      },
      "stonehearth:attributes": {}
   },

   didInsertElement: function() {
      this._super();
      var self = this;

      self.$("input").click(function() {
         radiant.call('radiant:play_sound', {'track' : 'stonehearth:sounds:ui:action_click' });
      });

      self._update();
   },

   click: function() {
      var self = this;
      self._selectRow();
   },

   _selectRow: function() {
      var self = this;
      var selected = self.$().hasClass('selected'); // Is this row already selected?
      if (selected) {
         return;
      }

      self.taskView.$('.row').removeClass('selected'); // Unselect everything in the parent view
      self.$().addClass('selected');

      self.taskView.setSelectedCitizen(self.get('model'));
   },

   _update: function() {
      var self = this;
      var citizenData = self.get('model');
      if (self.$() && citizenData) {
         // fixup row selection
         if (!self.$().hasClass('selected')) {
            var existingSelected = self.taskView.get('selected');
            if (!existingSelected) {
               // If no selected, select ourself if we're the first element
               if (self.get('index') == 0) {
                  self._selectRow();
               }
            } else {
               if (self.get('uri') == existingSelected.__self) {
                  self._selectRow();
               }
            }
         }

         // add tooltip
         var description = i18n.t(citizenData['unit_info'].description, {self: citizenData});
         self.$('.nameRow').tooltipster({content: description});
      }
   }.observes('model'),

    //When the attribute data changes, update the bars
   _setAttributeData: function() {
      Ember.run.scheduleOnce('afterRender', this, '_updateAttributes');
   }.observes('model.stonehearth:attributes' , 'model.stonehearth:buffs'),

   _updateAttributes: function() {
      var self = this;

      var healthPercent = Math.floor(self.get('model.stonehearth:attributes.attributes.health.user_visible_value') * 100 / self.get('model.stonehearth:attributes.attributes.max_health.user_visible_value'))
      var moralePercent = Math.floor(self.get('model.stonehearth:score.scores.happiness.score'));
      var expPercent = Math.floor(self.get('model.stonehearth:job.current_level_exp') * 100 / self.get('model.stonehearth:job.xp_to_next_lv'))

      self.set('model.health_bar_style', 'width: ' + healthPercent + '%');
      self.set('model.morale_bar_style', 'width: ' + moralePercent + '%');
      self.set('model.exp_bar_style', 'width: ' + expPercent + '%');
   },

   citizenId: function() {
      var self = this;

      var citizenData = self.get('model');
      if (!citizenData) {
         return undefined;
      }

      var match = citizenData.__self.match(/\/(\d+)$/)
      if (!match) {
         return undefined;
      }
      return parseInt(match[1]);
   }.property('model'),

   workOrderChecked : function(workOrder) {
      var self = this;

      var work_orders = self.get('taskView.model.work_orders');
      if (!work_orders) {
         return false;
      }
      var citizenId = self.get('citizenId')
      if (!citizenId) {
         return false;
      }
      return work_orders[workOrder].citizens[citizenId];
   },

   // Checked properties used to control whether or not the checkbox is checked
   haulChecked: function() {
      return this.workOrderChecked('haul');
   }.property('citizenId', 'taskView.model'),

   buildChecked: function() {
      return this.workOrderChecked('build');
   }.property('citizenId', 'taskView.model'),

   mineChecked: function() {
      return this.workOrderChecked('mine');
   }.property('citizenId', 'taskView.model'),

   jobChecked: function() {
      return this.workOrderChecked('job');
   }.property('citizenId', 'taskView.model'),

   // Id properties for identifying checkboxes
   haulId: function() {
      return "haul_" + this.get('citizenId');
   }.property('citizenId'),

   buildId: function() {
      return "build_" + this.get('citizenId');
   }.property('citizenId'),

   mineId: function() {
      return "mine_" + this.get('citizenId');
   }.property('citizenId'),

   jobId: function() {
      return "job_" + this.get('citizenId');
   }.property('citizenId'),

});
